// Copyright (c) 2016, Shinichiro Abe. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

part of ducene;

abstract class IndexDirectory {
  void outputIndex(String name, ListMultimap<String,int> index);
  void outputData(String name, ListMultimap<int,String> data);
  Map<String,ListMultimap<String,int>> inputIndex();
  Map<String,ListMultimap<int,String>> inputData();
}

class RAMIndexDirectory implements IndexDirectory {

  var _ramIndex = {};
  var _ramData = {};

  void outputIndex(String name, ListMultimap<String,int> index) {
    _ramIndex[name] = index;
  }

  void outputData(String name, ListMultimap<int,String> data) {
    _ramData[name] = data;
  }

  Map<String,ListMultimap<String,int>> inputIndex() => _ramIndex;
  Map<String,ListMultimap<int,String>> inputData() => _ramData;

}

class FSIndexDirectory implements IndexDirectory {

  final Directory _dir;

  FSIndexDirectory(this._dir) {
    if (_dir.existsSync()) _dir.deleteSync(recursive: true);
    if (!_dir.existsSync()) _dir.createSync(recursive: true);
  }

  void outputIndex(String name, ListMultimap<String,int> index) {
    File indexFile = new File('${_dir.path}/${name}.idx');
    if (!indexFile.existsSync()) indexFile.createSync();
    for (String key in index.keys) {
      String value = index[key].join(',');
      String line = '$key=>$value';
      indexFile.writeAsStringSync(line+'\n', mode: FileMode.APPEND, encoding: UTF8);
    }
  }

  void outputData(String name, ListMultimap<int,String> data) {
    File storeFile = new File('${_dir.path}/${name}.data');
    if (!storeFile.existsSync()) storeFile.createSync();
    for (int key in data.keys) {
      String value = data[key].join('\$');
      String line = '${key.toString()}=>${value}';
      storeFile.writeAsStringSync(line+'\n', mode: FileMode.APPEND, encoding: UTF8);
    }
  }

  Map<String,ListMultimap<String,int>> inputIndex() {
    var ret = {};
    for (String path in _getFileList('.idx')) {
      String name = _getName(path, '.idx');
      File indexFile = new File(path);
      List<String> lines = indexFile.readAsLinesSync();
      ListMultimap<String,int> invertedIndex = new ListMultimap();
      for (String line in lines) {
        List<String> kv = line.split('=>');
        String key = kv[0];
        List<int> values = kv[1].split(',').map((s) => int.parse(s)).toList();
        invertedIndex.addValues(key, values);
      }
      ret[name] = invertedIndex;
    }
    return ret;
  }

  Map<String,ListMultimap<int,String>> inputData() {
    var ret = {};
    for (String path in _getFileList('.data')) {
      String name = _getName(path, '.data');
      File dataFile = new File(path);
      List<String> lines = dataFile.readAsLinesSync();
      ListMultimap<int,String> storedData = new ListMultimap();
      for (String line in lines) {
        List<String> kv = line.split('=>');
        int key = int.parse(kv[0]);
        List<String> values = kv[1].split('\$');
        storedData.addValues(key, values);
      }
      ret[name] = storedData;
    }
    return ret;
  }

  Iterable _getFileList(String extension) =>
      _dir.listSync(recursive: false, followLinks: false)
          .map((e) => e.path)
          .where((path) => path.endsWith(extension));

  String _getName(String path, String extension) =>
      path.split('/').last.replaceAll(extension, '');

}